@@ -3,7 +3,7 @@ Base functionality of meshed
 """
 from collections import Counter
 from dataclasses import dataclass, field, fields
-from functools import partial
+from functools import partial, cached_property
 from typing import Callable, MutableMapping, Iterable, Union, Sized, Sequence
 
 from i2 import Sig, call_somewhat_forgivingly
@@ -335,6 +335,16 @@ class FuncNode:
         return isinstance(obj, cls)
 
 
+@dataclass
+class Mesh:
+    func_nodes: Iterable[FuncNode]
+
+    def synopsis_string(self, bind_info='values'):
+        return '\n'.join(
+            func_node.synopsis_string(bind_info) for func_node in self.func_nodes
+        )
+
+
 def validate_that_func_node_names_are_sane(func_nodes: Iterable[FuncNode]):
     """Assert that the names of func_nodes are sane.
     That is:
@@ -422,7 +432,7 @@ def get_init_params_of_instance(obj):
     return {k: v for k, v in vars(obj).items() if k in Sig(type(obj)).names}
 
 
-def ch_func_node_attrs(fn, **new_attrs_values):
+def ch_func_node_attrs(fn: FuncNode, **new_attrs_values):
     """Returns a copy of the func node with some of it's attributes changed
 
     >>> def plus(a, b):
@@ -457,6 +467,33 @@ def ch_func_node_attrs(fn, **new_attrs_values):
     return FuncNode(**fn_kwargs)
 
 
+def compare_signatures(func1, func2):
+    return Sig(func1) == Sig(func2)
+
+
+def raise_signature_mismatch_error(fn, func):
+    raise ValueError(
+        'You can only change the func of a FuncNode with a another func if the '
+        'signatures match.\n'
+        f'\t{fn=}\n'
+        f'\t{Sig(fn.func)=}\n'
+        f'\t{Sig(func)=}\n'
+    )
+
+
+def ch_func_node_func(
+    fn: FuncNode,
+    func: Callable,
+    *,
+    compare_func=compare_signatures,
+    alternative=raise_signature_mismatch_error,
+):
+    if compare_func(fn.func, func):
+        return ch_func_node_attrs(fn, func=func)
+    else:
+        return alternative(fn, func)
+
+
 def _keys_and_values_are_strings_validation(d: dict):
     for k, v in d.items():
         if not isinstance(k, str):
