@@ -405,7 +405,7 @@ def _topological_sort_helper(g, parent, visited, stack):
     visited.add(parent)  # Mark the current node as visited.
 
     # Recurse for all the vertices adjacent to this node
-    for child in g.get(parent, []):
+    for child in reversed(g.get(parent, [])):
         if child not in visited:
             _topological_sort_helper(g, child, visited, stack)
 
@@ -430,7 +430,7 @@ def topological_sort(g: Mapping):
     ... }
     >>>
     >>> list(topological_sort(g))
-    [0, 2, 4, 3, 1]
+    [0, 4, 2, 3, 1]
 
     Here's an ascii art of the graph, to verify that the topological sort is
     indeed as expected.
@@ -453,7 +453,7 @@ def topological_sort(g: Mapping):
 
     # Call the recursive helper function to accumulate topological sorts
     # starting from all vertices one by one
-    for parent in g:
+    for parent in reversed(g):
         if parent not in visited:
             # print(f"Processing {parent}")
             _topological_sort_helper(g, parent, visited, stack)
