@@ -162,6 +162,8 @@ from functools import partial
 
 
 from i2 import Sig, name_of_obj, partialx, double_up_as_factory
+from i2.signatures import name_of_obj
+
 
 from meshed.dag import DAG
 from meshed.base import FuncNode
@@ -561,3 +563,14 @@ def _ensure_name(name, src):
 #     visitor.visit(nodes)
 #
 #     return visitor.store
+
+
+def lined_dag(funcs):
+    dag = DAG(funcs)
+    if not funcs:
+        return dag
+    else:
+        names = [name_of_obj(func) for func in funcs]
+        pairs = zip(names, names[1:])
+        dag = dag.add_edges(pairs)
+        return dag
